/******** generated automatically from generate_source. See generate_source.c */
{"CursorLeft", NULL, "Cursor Left", NULL,"Moves the cursor one object left, altering the selection if any",G_CALLBACK (cursorleft_cb)},
{"MoveCursorLeft", NULL, "Move Cursor Left", NULL,"Moves the cursor one object left, without altering the selection",G_CALLBACK (movecursorleft_cb)},
{"CursorDown", NULL, "Cursor Down", NULL,"Moves the cursor one scale step down",G_CALLBACK (cursordown_cb)},
{"CursorUp", NULL, "Cursor Up", NULL,"Moves the cursor one scale step up",G_CALLBACK (cursorup_cb)},
{"CursorRight", NULL, "Cursor Right", NULL,"Moves the cursor one object right, altering the selection if any",G_CALLBACK (cursorright_cb)},
{"MoveCursorRight", NULL, "Move Cursor Right", NULL,"Moves the cursor one object right, without altering the selection",G_CALLBACK (movecursorright_cb)},
{"GoToMark", NULL, "To Mark", NULL,"Moves the cursor to the Mark without altering the selection",G_CALLBACK (goto_mark)},
{"SwapPointAndMark", NULL, "Swap Ends of Selection", NULL,"Swaps the active end of the selection",G_CALLBACK (swap_point_and_mark)},
{"GoToSelectionStart", NULL, "To Selection Start", NULL,"Moves the cursor to the first object in the selection without altering the selection. returns #f if no selection",G_CALLBACK (goto_selection_start)},
{"PushPosition", NULL, "Push Position", NULL,"Pushes the current cursor position onto a stack",G_CALLBACK (PushPosition)},
{"PopPosition", NULL, "Pop Position", NULL,"Pops a position from the stack of cursor positions, moving the cursor there",G_CALLBACK (PopPosition)},
{"PopPushPosition", NULL, "Pop and Push Position", NULL,"Pops a position from the stack of cursor positions, pushes the current position, then moves the cursor to the popped position",G_CALLBACK (PopPushPosition)},
{"ToggleReduceToDrawingArea", NULL, "Hide/Show Menus", NULL,"Hides/Shows menus, panes etc. The ones shown are those checked in the view menu.",G_CALLBACK (ToggleReduceToDrawingArea)},
{"StaffUp", NULL, "Staff Up", NULL,"Moves the cursor to the staff above, extending selection if any",G_CALLBACK (staffup_cb)},
{"StaffDown", NULL, "Staff Down", NULL,"Moves the cursor to the staff below, extending selection if any",G_CALLBACK (staffdown_cb)},
{"MoveToStaffUp", NULL, "Move to Staff Up", NULL,"Moves the cursor to the staff above without altering selection",G_CALLBACK (movetostaffup_cb)},
{"MoveToStaffDown", NULL, "Move to Staff Down", NULL,"Moves the cursor to the staff below  without altering selection",G_CALLBACK (movetostaffdown_cb)},
{"MeasureLeft", NULL, "Measure Left", NULL,"Moves the cursor to the first object in the next measure, extending selection if any",G_CALLBACK (measureleft_cb)},
{"MeasureRight", NULL, "Measure Right", NULL,"Moves the cursor to the first object in the previous measure, extending selection if any",G_CALLBACK (measureright_cb)},
{"MoveToMeasureLeft", NULL, "Move to Measure Left", NULL,"Moves the cursor to the first object in the next measure leaving selection, if any, unchanged",G_CALLBACK (movetomeasureleft_cb)},
{"MoveToMeasureRight", NULL, "Move to Measure Right", NULL,"Moves the cursor to the first object in the previous measureleaving selection, if any, unchanged",G_CALLBACK (movetomeasureright_cb)},
{"A", NULL, "A", NULL,"Append/Edit  A",G_CALLBACK (go_to_A_key_cb)},
{"B", NULL, "B", NULL,"Append/Edit  B",G_CALLBACK (go_to_B_key_cb)},
{"C", NULL, "C", NULL,"Append/Edit  C",G_CALLBACK (go_to_C_key_cb)},
{"D", NULL, "D", NULL,"Append/Edit  D",G_CALLBACK (go_to_D_key_cb)},
{"E", NULL, "E", NULL,"Append/Edit  E",G_CALLBACK (go_to_E_key_cb)},
{"F", NULL, "F", NULL,"Append/Edit  F",G_CALLBACK (go_to_F_key_cb)},
{"G", NULL, "G", NULL,"Append/Edit  G",G_CALLBACK (go_to_G_key_cb)},
{"OctaveUp", NULL, "Octave Up", NULL,"Octave Up",G_CALLBACK (octave_up_key_cb)},
{"OctaveDown", NULL, "Octave Down", NULL,"Octave Down",G_CALLBACK (octave_down_key_cb)},
{"WholeNote", NULL, "WholeNote", NULL,"Insert \xF0\x9D\x85\x9D",G_CALLBACK (insert_chord_0key_cb)},
{"HalfNote", NULL, "HalfNote", NULL,"Insert \xF0\x9D\x85\x9E",G_CALLBACK (insert_chord_1key_cb)},
{"QuarterNote", NULL, "QuarterNote", NULL,"Insert \xF0\x9D\x85\x9F",G_CALLBACK (insert_chord_2key_cb)},
{"EighthNote", NULL, "EighthNote", NULL,"Insert \xF0\x9D\x85\xA0",G_CALLBACK (insert_chord_3key_cb)},
{"SixteenthNote", NULL, "SixteenthNote", NULL,"Insert \xF0\x9D\x85\xA1",G_CALLBACK (insert_chord_4key_cb)},
{"ThirtysecondNote", NULL, "ThirtysecondNote", NULL,"Insert \xF0\x9D\x85\xA2",G_CALLBACK (insert_chord_5key_cb)},
{"SixtyfourthNote", NULL, "SixtyfourthNote", NULL,"Insert \xF0\x9D\x85\xA3",G_CALLBACK (insert_chord_6key_cb)},
{"OneHundredTwentyEighthNote", NULL, "OneHundredTwentyEighthNote", NULL,"Insert \xF0\x9D\x85\xA4",G_CALLBACK (insert_chord_7key_cb)},
{"TwoHundredFiftySixthNote", NULL, "TwoHundredFiftySixthNote", NULL,"Insert \xF0\x9D\x85\xA5",G_CALLBACK (insert_chord_8key_cb)},
{"InsertWholeRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBB</span>", NULL,"Insert \xF0\x9D\x84\xBB rest",G_CALLBACK (insert_rest_0key_cb)},
{"InsertHalfRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBC</span>", NULL,"Insert \xF0\x9D\x84\xBC rest",G_CALLBACK (insert_rest_1key_cb)},
{"InsertQuarterRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBD</span>", NULL,"Insert \xF0\x9D\x84\xBD rest",G_CALLBACK (insert_rest_2key_cb)},
{"InsertEighthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBE</span>", NULL,"Insert \xF0\x9D\x84\xBE rest",G_CALLBACK (insert_rest_3key_cb)},
{"InsertSixteenthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBF</span>", NULL,"Insert \xF0\x9D\x84\xBF rest",G_CALLBACK (insert_rest_4key_cb)},
{"InsertThirtysecondRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x85\x80</span>", NULL,"Insert \xF0\x9D\x85\x80 rest",G_CALLBACK (insert_rest_5key_cb)},
{"InsertSixtyfourthRest", NULL, "<span font_desc=\"Denemo\">\xF0\x9D\x85\x81</span>", NULL,"Insert \xF0\x9D\x85\x81 rest",G_CALLBACK (insert_rest_6key_cb)},
{"InsertBlankWholeNote", NULL, "InsertBlankWholeNote", NULL,"Insert a non-printing \xF0\x9D\x84\xBB rest",G_CALLBACK (insert_blankchord_0key_cb)},
{"InsertBlankHalfNote", NULL, "InsertBlankHalfNote", NULL,"Insert a non-printing \xF0\x9D\x84\xBC rest",G_CALLBACK (insert_blankchord_1key_cb)},
{"InsertBlankQuarterNote", NULL, "InsertBlankQuarterNote", NULL,"Insert a non-printing \xF0\x9D\x84\xBD rest",G_CALLBACK (insert_blankchord_2key_cb)},
{"InsertBlankEighthNote", NULL, "InsertBlankEighthNote", NULL,"Insert a non-printing \xF0\x9D\x84\xBE rest",G_CALLBACK (insert_blankchord_3key_cb)},
{"InsertBlankSixteenthNote", NULL, "InsertBlankSixteenthNote", NULL,"Insert a non-printing \xF0\x9D\x84\xBF rest",G_CALLBACK (insert_blankchord_4key_cb)},
{"InsertBlankThirtysecondNote", NULL, "InsertBlankThirtysecondNote", NULL,"Insert a non-printing \xF0\x9D\x85\x80 rest",G_CALLBACK (insert_blankchord_5key_cb)},
{"InsertBlankSixtyfourthNote", NULL, "InsertBlankSixtyfourthNote", NULL,"Insert a non-printing \xF0\x9D\x85\x81 rest",G_CALLBACK (insert_blankchord_6key_cb)},
{"InsertBlankOneHundredTwentyEighthNote", NULL, "InsertBlankOneHundredTwentyEighthNote", NULL,"Insert a non-printing \xF0\x9D\x85\x82 rest",G_CALLBACK (insert_blankchord_7key_cb)},
{"InsertBlankTwoHundredFiftySixthNote", NULL, "InsertBlankTwoHundredFiftySixthNote", NULL,"Insert a non-printing \xF0\x9D\x85\x83 rest",G_CALLBACK (insert_blankchord_8key_cb)},
{"ToggleRestMode", NULL, "Toggle Rest Mode", NULL,"No Tooltip yet",G_CALLBACK (rest_toggle_key_cb)},
{"ToggleBlankMode", NULL, "Toggle Blank Mode", NULL,"No Tooltip yet",G_CALLBACK (toggle_blank_cb)},
{"InsertDuplet", NULL, "Insert Duplet", NULL,"No Tooltip yet",G_CALLBACK (insert_duplet_cb)},
{"InsertTriplet", NULL, "Insert Triplet", NULL,"No Tooltip yet",G_CALLBACK (insert_triplet_cb)},
{"StartTriplet", NULL, "Start Triplet", NULL,"No Tooltip yet",G_CALLBACK (start_triplet_cb)},
{"EndTuplet", NULL, "End Tuplet", NULL,"No Tooltip yet",G_CALLBACK (end_tuplet_cb)},
{"InsertQuadtuplet", NULL, "Insert Quadtuplet", NULL,"No Tooltip yet",G_CALLBACK (insert_quadtuplet_cb)},
{"InsertQuintuplet", NULL, "Insert Quintuplet", NULL,"No Tooltip yet",G_CALLBACK (insert_quintuplet_cb)},
{"InsertSextuplet", NULL, "Insert Sextuplet", NULL,"No Tooltip yet",G_CALLBACK (insert_sextuplet_cb)},
{"InsertSeptuplet", NULL, "Insert Septuplet", NULL,"No Tooltip yet",G_CALLBACK (insert_septuplet_cb)},
{"AddNoteToChord", NULL, "Add note", NULL,"Add a note to the current chord\nThe cursor position determines which note to add",G_CALLBACK (add_tone_key_cb)},
{"RemoveNoteFromChord", NULL, "Remove note", NULL,"Remove a note from the current chord",G_CALLBACK (remove_tone_key_cb)},
{"Sharpen", NULL, "Sharpen", NULL,"No Tooltip yet",G_CALLBACK (sharpen_key_cb)},
{"Flatten", NULL, "Flatten", NULL,"No Tooltip yet",G_CALLBACK (flatten_key_cb)},
{"PendingSharpen", NULL, "Sharpen Next Note", NULL,"Increases the sharpness of the next entered note. The status bar shows the current state.",G_CALLBACK (pending_sharpen_cb)},
{"PendingFlatten", NULL, "Flatten Next Note", NULL,"Increases the flatness of the next entered note. The status bar shows the current state.",G_CALLBACK (pending_flatten_cb)},
{"StemUp", NULL, "StemUp", NULL,"Alters a StemNeutral object to stem up.",G_CALLBACK (stem_up_cb)},
{"StemDown", NULL, "StemDown", NULL,"Alters a StemNeutral object to stem down.",G_CALLBACK (stem_down_cb)},
{"AddDot", NULL, "Add Dot", NULL,"No Tooltip yet",G_CALLBACK (add_dot_key_cb)},
{"RemoveDot", NULL, "Remove Dot", NULL,"No Tooltip yet",G_CALLBACK (remove_dot_key_cb)},
{"InsertTiedNote", NULL, "Tied note", NULL,"Inserts a duplicate of the current note, tied",G_CALLBACK (tie_notes_key_cb)},
{"ToggleTie", NULL, "Toggle Tie", NULL,"Ties/unties the note at the cursor",G_CALLBACK (toggle_tie)},
{"DeleteObject", NULL, "Delete Object", NULL,"Delete the object at the cursor",G_CALLBACK (deleteobject_cb)},
{"DeletePreviousObject", NULL, "Delete Previous Object", NULL,"Delete to the left of the cursor.",G_CALLBACK (deletepreviousobject_cb)},
{"InsertMeasure", NULL, "Insert Measure Before", NULL,"Insert a blank measure before the current one (in all staffs)",G_CALLBACK (insert_measure_key_cb)},
{"AddMeasure", NULL, "Insert Measure After", NULL,"Insert a blank measure after the current one (in all staffs)",G_CALLBACK (addmeasureafter_cb)},
{"InsertMeasureBefore", NULL, "Staff Insert Measure Before", NULL,"Insert a blank measure before the current one (in current staff)",G_CALLBACK (insertmeasurebefore_cb)},
{"InsertMeasureAfter", NULL, "Staff Insert Measure After", NULL,"Insert a blank measure in current staff after the current measure",G_CALLBACK (insertmeasureafter_cb)},
{"AppendMeasure", NULL, "Staff Append Measure", NULL,"No Tooltip yet",G_CALLBACK (append_measure_key_cb)},
{"DeleteMeasure", NULL, "Staff Delete Measure", NULL,"Delete the current measure in this staff, leaving the staff short",G_CALLBACK (deletemeasure_cb)},
{"DeleteMeasureAllStaffs", NULL, "Delete Measure All Staffs", NULL,"Delete the current measure in all staffs",G_CALLBACK (deletemeasureallstaffs_cb)},
{"ShrinkMeasures", NULL, "Shrink Measure", NULL,"No Tooltip yet",G_CALLBACK (adjust_measure_less_width_key_cb)},
{"WidenMeasures", NULL, "Widen Measures", NULL,"No Tooltip yet",G_CALLBACK (adjust_measure_more_width_key_cb)},
{"ShorterStaffs", NULL, "Shorter Staffs", NULL,"No Tooltip yet",G_CALLBACK (adjust_staff_less_height_key_cb)},
{"TallerStaffs", NULL, "Taller Staffs", NULL,"No Tooltip yet",G_CALLBACK (adjust_staff_more_height_key_cb)},
{"InsertTrebleClef", NULL, "New Treble Clef", NULL,"No Tooltip yet",G_CALLBACK (newcleftreble_cb)},
{"InsertBassClef", NULL, "New Bass Clef", NULL,"No Tooltip yet",G_CALLBACK (newclefbass_cb)},
{"Insertg8clef", NULL, "New G8 Clef", NULL,"No Tooltip yet",G_CALLBACK (newclefg8_cb)},
{"InsertAltoClef", NULL, "New Alto Clef", NULL,"No Tooltip yet",G_CALLBACK (newclefalto_cb)},
{"InsertTenorClef", NULL, "New Tenor Clef", NULL,"No Tooltip yet",G_CALLBACK (newcleftenor_cb)},
{"InsertSopranoClef", NULL, "New Soprano Clef", NULL,"No Tooltip yet",G_CALLBACK (newclefsoprano_cb)},
{"SetInitialTrebleClef", NULL, "Set Treble Clef", NULL,"No Tooltip yet",G_CALLBACK (setcleftreble_cb)},
{"SetInitialBassClef", NULL, "Set Bass Clef", NULL,"No Tooltip yet",G_CALLBACK (setclefbass_cb)},
{"SetInitialg8clef", NULL, "Set G8 Clef", NULL,"No Tooltip yet",G_CALLBACK (setclefg8_cb)},
{"SetInitialAltoClef", NULL, "Set Alto Clef", NULL,"No Tooltip yet",G_CALLBACK (setclefalto_cb)},
{"SetInitialTenorClef", NULL, "Set Tenor Clef", NULL,"No Tooltip yet",G_CALLBACK (setcleftenor_cb)},
{"SetInitialSopranoClef", NULL, "Set Soprano Clef", NULL,"No Tooltip yet",G_CALLBACK (setclefsoprano_cb)},
{"Insert22Time", NULL, "Insert 2/2 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig22_cb)},
{"Insert32Time", NULL, "Insert 3/2 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig32_cb)},
{"Insert42Time", NULL, "Insert 4/2 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig42_cb)},
{"Insert44Time", NULL, "Insert 4/4 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig44_cb)},
{"Insert34Time", NULL, "Insert 3/4 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig34_cb)},
{"Insert24Time", NULL, "Insert 2/4 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig24_cb)},
{"Insert64Time", NULL, "Insert 6/4 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig64_cb)},
{"Insert38Time", NULL, "Insert 3/8 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig38_cb)},
{"Insert68Time", NULL, "Insert 6/8 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig68_cb)},
{"Insert128Time", NULL, "Insert 12/8 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig128_cb)},
{"Insert98Time", NULL, "Insert 9/8 Time", NULL,"No Tooltip yet",G_CALLBACK (newtimesig98_cb)},
{"Set22Time", NULL, "Set 2/2 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig22_cb)},
{"Set32Time", NULL, "Set 3/2 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig32_cb)},
{"Set42Time", NULL, "Set 4/2 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig42_cb)},
{"Set44Time", NULL, "Set 4/4 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig44_cb)},
{"Set34Time", NULL, "Set 3/4 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig34_cb)},
{"Set24Time", NULL, "Set 2/4 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig24_cb)},
{"Set64Time", NULL, "Set 6/4 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig64_cb)},
{"Set38Time", NULL, "Set 3/8 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig38_cb)},
{"Set68Time", NULL, "Set 6/8 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig68_cb)},
{"Set128Time", NULL, "Set 12/8 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig128_cb)},
{"Set98Time", NULL, "Set 9/8 Time", NULL,"No Tooltip yet",G_CALLBACK (settimesig98_cb)},
{"InsertCmaj", NULL, "Insert Cmaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysigcmaj_cb)},
{"InsertGmaj", NULL, "Insert Gmaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysiggmaj_cb)},
{"InsertDmaj", NULL, "Insert Dmaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysigdmaj_cb)},
{"InsertAmaj", NULL, "Insert Amaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysigamaj_cb)},
{"InsertEmaj", NULL, "Insert Emaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysigemaj_cb)},
{"InsertBmaj", NULL, "Insert Bmaj", NULL,"No Tooltip yet",G_CALLBACK (newkeysigbmaj_cb)},
{"InsertFSharpmaj", NULL, "Insert F# Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigfsharpmaj_cb)},
{"InsertCSharpmaj", NULL, "Insert C# Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigcsharpmaj_cb)},
{"InsertFmaj", NULL, "Insert F Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigfmaj_cb)},
{"InsertBflatmaj", NULL, "Insert Bb Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigbflatmaj_cb)},
{"InsertEflatmaj", NULL, "Insert Eb Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigeflatmaj_cb)},
{"InsertAflatmaj", NULL, "Insert Ab Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigaflatmaj_cb)},
{"InsertDflatmaj", NULL, "Insert Db Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigdflatmaj_cb)},
{"InsertGflatmaj", NULL, "Insert Gb Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysiggflatmaj_cb)},
{"InsertCflatmaj", NULL, "Insert Cb Major", NULL,"No Tooltip yet",G_CALLBACK (newkeysigcflatmaj_cb)},
{"InsertAmin", NULL, "Insert A Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigamin_cb)},
{"InsertEmin", NULL, "Insert E Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigemin_cb)},
{"InsertBmin", NULL, "Insert B Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigbmin_cb)},
{"InsertFSharpmin", NULL, "Insert F# Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigfsharpmin_cb)},
{"InsertCSharpmin", NULL, "Insert C# Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigcsharpmin_cb)},
{"InsertGSharpmin", NULL, "Insert G# Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysiggsharpmin_cb)},
{"InsertDSharpmin", NULL, "Insert D# Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigdsharpmin_cb)},
{"InsertASharpmin", NULL, "Insert A# Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigasharpmin_cb)},
{"InsertDmin", NULL, "Insert D Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigdmin_cb)},
{"InsertGmin", NULL, "Insert G Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysiggmin_cb)},
{"InsertCmin", NULL, "Insert C Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigcmin_cb)},
{"InsertFmin", NULL, "Insert F Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigfmin_cb)},
{"InsertBflatmin", NULL, "Insert Bb Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigbflatmin_cb)},
{"InsertEflatmin", NULL, "Insert Eb Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigeflatmin_cb)},
{"InsertAflatmin", NULL, "Insert Ab Minor", NULL,"No Tooltip yet",G_CALLBACK (newkeysigaflatmin_cb)},
{"SetInitialCmaj", NULL, "Set Initial Keysig to C Major", NULL,"No Tooltip yet",G_CALLBACK (setkeysigcmaj_cb)},
{"SetInitialGmaj", NULL, "Set Initial Keysig to G Major", NULL,"No Tooltip yet",G_CALLBACK (setkeysiggmaj_cb)},
{"SetInitialDmaj", NULL, "Set D Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigdmaj_cb)},
{"SetInitialAmaj", NULL, "Set A Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigamaj_cb)},
{"SetInitialEmaj", NULL, "Set E Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigemaj_cb)},
{"SetInitialBmaj", NULL, "Set B Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigbmaj_cb)},
{"SetInitialFSharpmaj", NULL, "Set F# Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigfsharpmaj_cb)},
{"SetInitialCSharpmaj", NULL, "Set C# Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigcsharpmaj_cb)},
{"SetInitialFmaj", NULL, "Set F Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigfmaj_cb)},
{"SetInitialBflatmaj", NULL, "Set Bb Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigbflatmaj_cb)},
{"SetInitialEflatmaj", NULL, "Set Eb Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigeflatmaj_cb)},
{"SetInitialAflatmaj", NULL, "Set Ab Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigaflatmaj_cb)},
{"SetInitialDflatmaj", NULL, "Set Db Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigdflatmaj_cb)},
{"SetInitialGflatmaj", NULL, "Set Gb Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysiggflatmaj_cb)},
{"SetInitialCflatmaj", NULL, "Set Cb Major as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigcflatmaj_cb)},
{"SetInitialAmin", NULL, "Set A Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigamin_cb)},
{"SetInitialEmin", NULL, "Set E Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigemin_cb)},
{"SetInitialBmin", NULL, "Set B Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigbmin_cb)},
{"SetInitialFSharpmin", NULL, "Set F# Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigfsharpmin_cb)},
{"SetInitialCSharpmin", NULL, "Set C# Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigcsharpmin_cb)},
{"SetInitialGSharpmin", NULL, "Set G# Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysiggsharpmin_cb)},
{"SetInitialDSharpmin", NULL, "Set D# Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigdsharpmin_cb)},
{"SetInitialASharpmin", NULL, "Set A# Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigasharpmin_cb)},
{"SetInitialDmin", NULL, "Set D Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigdmin_cb)},
{"SetInitialGmin", NULL, "Set G Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysiggmin_cb)},
{"SetInitialCmin", NULL, "Set C Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigcmin_cb)},
{"SetInitialFmin", NULL, "Set F Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigfmin_cb)},
{"SetInitialBflatmin", NULL, "Set Bb Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigbflatmin_cb)},
{"SetInitialEflatmin", NULL, "Set Eb Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigeflatmin_cb)},
{"SetInitialAflatmin", NULL, "Set Ab Minor as Initial Keysig", NULL,"No Tooltip yet",G_CALLBACK (setkeysigaflatmin_cb)},
{"SetMark", NULL, "Set Mark", NULL,"Sets the start point for a selection,\nthe end point of the selection is unaltered",G_CALLBACK (set_mark_cb)},
{"UnsetMark", NULL, "Unset Mark", NULL,"Gets rid of the selection.",G_CALLBACK (unset_mark_cb)},
{"SetPoint", NULL, "Set Point", NULL,"Extends the selection to the current cursor position",G_CALLBACK (set_point_cb)},
{"ToggleBeginSlur", NULL, "Begin Slur", NULL,"Insert/delete begin slur on this note",G_CALLBACK (toggle_begin_slur_cb)},
{"ToggleEndSlur", NULL, "End Slur", NULL,"Insert/delete end slur on this note",G_CALLBACK (toggle_end_slur_cb)},
{"ToggleStartCrescendo", NULL, "Start Crescendo", NULL,"No Tooltip yet",G_CALLBACK (toggle_start_crescendo_cb)},
{"ToggleEndCrescendo", NULL, "End Crescendo", NULL,"No Tooltip yet",G_CALLBACK (toggle_end_crescendo_cb)},
{"ToggleStartDiminuendo", NULL, "Start Diminuendo", NULL,"No Tooltip yet",G_CALLBACK (toggle_start_diminuendo_cb)},
{"ToggleEndDiminuendo", NULL, "End Diminuendo", NULL,"No Tooltip yet",G_CALLBACK (toggle_end_diminuendo_cb)},
{"ToggleAccent", NULL, "Accent (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_accent_cb)},
{"ToggleFermata", NULL, "Fermata (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_fermata_cb)},
{"ToggleStaccato", NULL, "Staccato (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_staccato_cb)},
{"ToggleTenuto", NULL, "Tenuto (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_tenuto_cb)},
{"ToggleTrill", NULL, "Trill  (Off/On", NULL,"No Tooltip yet",G_CALLBACK (add_trill_cb)},
{"ToggleTurn", NULL, "Turn (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_turn_cb)},
{"ToggleMordent", NULL, "Mordent (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_mordent_cb)},
{"ToggleStaccatissimo", NULL, "Staccatissimo  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_staccatissimo_cb)},
{"ToggleCoda", NULL, "Coda (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_coda_cb)},
{"ToggleFlageolet", NULL, "Flageolet  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_flageolet_cb)},
{"ToggleOpen", NULL, "Open (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_open_cb)},
{"TogglePrallMordent", NULL, "Prall Mordent  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_prallmordent_cb)},
{"TogglePrallPrall", NULL, "Prall Prall  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_prallprall_cb)},
{"TogglePrall", NULL, "Prall (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_prall_cb)},
{"ToggleReverseTurn", NULL, "Reverse Turn  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_reverseturn_cb)},
{"ToggleSegno", NULL, "Segno  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_segno_cb)},
{"ToggleSforzato", NULL, "Sforzato  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_sforzato_cb)},
{"ToggleStopped", NULL, "Stopped  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_stopped_cb)},
{"ToggleThumb", NULL, "Thumb  (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_thumb_cb)},
{"ToggleUpprall", NULL, "Up Prall (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_upprall_cb)},
{"ToggleArpeggio", NULL, "Apreggio (Off/On)", NULL,"No Tooltip yet",G_CALLBACK (add_arpeggio_cb)},
{"SetGrace", NULL, "Set Grace", NULL,"No Tooltip yet",G_CALLBACK (set_grace_cb)},
{"ToggleGrace", NULL, "Grace Note Off/On", NULL,"Makes the note at the cursor a grace note, if it is one, makes it normal",G_CALLBACK (toggle_grace)},
{"ForceCaution", NULL, "Force Cautionary Accidental", NULL,"No Tooltip yet",G_CALLBACK (force_cautionary_cb)},
{"ChangePitch", NULL, "Change Pitch", NULL,"No Tooltip yet",G_CALLBACK (change_pitch_cb)},
{"InsertRhythm", NULL, "Insert Snippet", NULL,"No Tooltip yet",G_CALLBACK (insert_rhythm_pattern_cb)},
{"NextRhythm", NULL, "Next Snippet", NULL,"Make next snippet\nthe current snippet.\nNotes entered will follow the rhythmic pattern of this snippet",G_CALLBACK (nextrhythm_cb)},
{"AppendMeasureAllStaffs", NULL, "Append Measure All Staffs", NULL,"Appends a blank measure to every staff in this movement",G_CALLBACK (append_measure_score_cb)},
{"ExecuteScheme", NULL, "Execute Scheme", NULL,"Execute the scheme code from the scripting window",G_CALLBACK (execute_scheme)},
{"FileMenu", NULL, "File", NULL,"Creating, saving, loading, displaying and printing musical scores"},
{"Bookmarks", NULL, "Bookmarks", NULL,"Creating, saving places in musical scores"},
{"ModeMenu", NULL, "Mode", NULL,"Different keyboard and MIDI entry modes"},
{"EditMenu", NULL, "Edit", NULL,"General editing commands"},
{"ViewMenu", NULL, "View", NULL,"Control which tools are to be shown"},
{"StaffMenu", NULL, "Staffs/Voices", NULL,"Staffs and voices"},
{"Voices", NULL, "Voices", NULL,"Voices"},
{"MovementMenu", NULL, "Movements", NULL,"Movements in a score"},
{"HelpMenu", NULL, "Help", NULL,"Help with denemo"},
{"InputMenu", NULL, "Input", NULL,"input audio or midi"},
{"SharpenEnharmonicSet", NULL, "Shift Accidentals Sharpwise", NULL,"Shifts the set of accidentals one step sharper",G_CALLBACK (set_sharper)},
{"FlattenEnharmonicSet", NULL, "Shift Accidentals Flatwise", NULL,"Shifts the set of accidentals one step flatter",G_CALLBACK (set_flatter)},
{"PlaybackMenu", NULL, "Playback", NULL,"Playing the music through midi file"},
{"New", GTK_STOCK_NEW, "New File", NULL,"Start a new musical score",G_CALLBACK (file_newwrapper)},
{"OpenMenu", GTK_STOCK_OPEN, "Open", NULL,"Open a file containing a music score for editing"},
{"Open", GTK_STOCK_OPEN, "Open", NULL,"Open a file containing a music score for editing",G_CALLBACK (file_open_with_check)},
{"ImportLilypond", GTK_STOCK_OPEN, "Import Lilypond", NULL,"Import a Lilypond file",G_CALLBACK (file_import_lilypond_with_check)},
{"ImportMidi", GTK_STOCK_OPEN, "Import Midi", NULL,"Import a Midi file",G_CALLBACK (file_import_midi_with_check)},
{"ImportMusicXml", GTK_STOCK_OPEN, "Import MusicXml", NULL,"Import a MusicXml file",G_CALLBACK (file_import_musicxml_with_check)},
{"AddStaffs", GTK_STOCK_OPEN, "Add Staffs", NULL,"Add staffs from a Denemo file",G_CALLBACK (file_add_staffs)},
{"AddMovements", GTK_STOCK_OPEN, "Add Movement", NULL,"Add movements from a Denemo file",G_CALLBACK (file_add_movements)},
{"MovementProps", GTK_STOCK_PROPERTIES, "Change Properties", NULL,"Change properties of this movement",G_CALLBACK (movement_props_dialog)},
{"OpenNewWindow", GTK_STOCK_OPEN, "Open In New", NULL,"Open a file containing a music score for editing in a separate working area (tab",G_CALLBACK (openinnew)},
{"Save", GTK_STOCK_SAVE, "Save", NULL,"Save the score",G_CALLBACK (file_savewrapper)},
{"SaveAs", GTK_STOCK_SAVE_AS, "Save As", NULL,"Save the score under a new name",G_CALLBACK (file_saveaswrapper)},
{"SaveCopy", GTK_STOCK_SAVE_AS, "Create Copy", NULL,"Save a copy of the score",G_CALLBACK (file_copy_save)},
{"OpenTemplate", GTK_STOCK_OPEN, "Open Template", NULL,"Start a new score from a built-in template file",G_CALLBACK (system_template_open_with_check)},
{"OpenExample", GTK_STOCK_OPEN, "Open Example", NULL,"Start a new score from a built-in example",G_CALLBACK (system_example_open_with_check)},
{"OpenMyTemplate", GTK_STOCK_OPEN, "Open Custom Template", NULL,"Start a new score from one of your own template files",G_CALLBACK (local_template_open_with_check)},
{"SaveTemplate", GTK_STOCK_SAVE_AS, "Save Template", NULL,"Save the score as a template for re-use as a starting point for new scores",G_CALLBACK (template_save)},
{"NewWindow", NULL, "New Tab", NULL,"Create working area (tab with an empty score in it)",G_CALLBACK (newview)},
{"InsertMovementBefore", NULL, "Insert Movement Before", NULL,"Insert a new movement before the current one",G_CALLBACK (insert_movement_before)},
{"InsertMovementAfter", NULL, "Insert Movement After", NULL,"Insert a new movement after the current one",G_CALLBACK (insert_movement_after)},
{"NewMovement", NULL, "New Movement", NULL,"Create a new movement, usign any default template",G_CALLBACK (append_new_movement)},
{"SaveMenu", GTK_STOCK_SAVE, "Save", NULL,"Save"},
{"SaveParts", GTK_STOCK_SAVE_AS, "Save Parts", NULL,"Save Parts: each staff becomes a file in lilypond format",G_CALLBACK (file_savepartswrapper)},
{"ExportMUDELA", GTK_STOCK_SAVE_AS, "Export Lilypond", NULL,"Export the score as a lilypond file",G_CALLBACK (export_mudela_action)},
{"ExportPDF", GTK_STOCK_SAVE_AS, "Export PDF", NULL,"Export the score as a PDF document file",G_CALLBACK (export_pdf_action)},
{"ExportPNG", GTK_STOCK_SAVE_AS, "Export PNG", NULL,"Export the score as a PNG image file",G_CALLBACK (export_png_action)},
{"ExportMIDI", GTK_STOCK_SAVE_AS, "Export MIDI", NULL,"Export the score as a MIDI file",G_CALLBACK (export_midi_action)},
{"ConfigureScore", GTK_STOCK_PROPERTIES, "Score Wizard", NULL,"Start up a wizard to create a new score. This allows you to set various properties of the score",G_CALLBACK (scorewizard)},
{"PrintMenu", GTK_STOCK_PRINT, "Print", NULL,"Displays the final finished score"},
{"PrintPreview", GTK_STOCK_PRINT_PREVIEW, "Print Preview", NULL,"Displays the final finished score in your pdf viewer",G_CALLBACK (printpreview_cb)},
{"PrintSelection", GTK_STOCK_PRINT, "Print Selection", NULL,"Displays selected music from score in your pdf viewer",G_CALLBACK (printselection_cb)},
{"PrintExcerptPreview", GTK_STOCK_PRINT_PREVIEW, "Print Excerpt", NULL,"Displays a musical excerpt in your image viewer",G_CALLBACK (printexcerptpreview_cb)},
{"Print", GTK_STOCK_PRINT, "Print", NULL,"Displays the final finished score in a pdf viewer. From this you can print the file using the print command of the viewer",G_CALLBACK (printall_cb)},
{"PrintPart", GTK_STOCK_PRINT, "Print Part", NULL,"Displays the final finished score for the current part (that is current staff",G_CALLBACK (printpart_cb)},
{"Close", GTK_STOCK_CLOSE, "Close Score", NULL,"Close the current score. Other windows will stay open",G_CALLBACK (close_gui_with_check)},
{"Quit", GTK_STOCK_QUIT, "Quit", NULL,"Quit the Denemo program",G_CALLBACK (closewrapper)},
{"Undo", GTK_STOCK_UNDO, "Undo", NULL,"Undoes one (more) step of your edits to the current score.",G_CALLBACK (undowrapper)},
{"Redo", GTK_STOCK_REDO, "Redo", NULL,"Redoes the next of the steps you have Undone",G_CALLBACK (redowrapper)},
{"Select", NULL, "Select", NULL,"Selecting stretches of notes"},
{"ExtendSelect", NULL, "Extend Selection", NULL,"Extend the selection"},
{"Copy", GTK_STOCK_COPY, "Copy", NULL,"Copy",G_CALLBACK (copywrapper)},
{"Cut", GTK_STOCK_CUT, "Cut", NULL,"Cut",G_CALLBACK (cutwrapper)},
{"Paste", GTK_STOCK_PASTE, "Paste", NULL,"Paste the selected music",G_CALLBACK (pastewrapper)},
{"PasteClipboard", GTK_STOCK_PASTE, "Paste LilyPond notes", NULL,"Paste LilyPond notes from the text clipboard",G_CALLBACK (paste_clipboard)},
{"ScoreProperties", GTK_STOCK_PROPERTIES, "Score Properties", NULL,"Change some of the properties of the current score. This will start up a dialog window",G_CALLBACK (score_properties_dialog)},
{"SaveSelection", NULL, "Save Selection", NULL,"Save the selected music. Not sure if this is working",G_CALLBACK (saveselwrapper)},
{"Preferences", GTK_STOCK_PREFERENCES, "Change Preferences", NULL,"Set and save your preferences for how Denemo operates on startup. Edit .denemo/denemorc for missing ones",G_CALLBACK (preferences_change)},
{"KeyBindings", NULL, "Customize Commands, Shortcuts ...", NULL,"Set actions to take in response to keypresses"},
{"SaveAccels", GTK_STOCK_SAVE, "Save Command Set", NULL,"Save the current commands and keyboard shortcuts as the default",G_CALLBACK (save_default_keymap_file_wrapper)},
{"CommandManagement", NULL, "Manage Command Set", NULL,"View help, change and save keyboard shortcuts",G_CALLBACK (configure_keyboard_dialog)},
{"SwapStaffs", NULL, "Swap Staffs", NULL,"Swap this staff with the one higher up. Note this actually swaps voices.",G_CALLBACK (swapstaffs)},
{"SplitVoices", NULL, "Split Voices", NULL,"Split off the next voice as a separate staff",G_CALLBACK (splitstaffs)},
{"JoinVoices", NULL, "Join Voices", NULL,"Merge this staff as a voice on the previous staff",G_CALLBACK (joinstaffs)},
{"SwapMovements", NULL, "Swap Movements", NULL,"Swap this movement with the one before",G_CALLBACK (swapmovements)},
{"VoiceUp", NULL, "Voice Up", NULL,"Go to the higher numbered voice on staff, extending selection if any",G_CALLBACK (voiceup_cb)},
{"VoiceDown", NULL, "Voice Down", NULL,"Go to the lower numbered voice on this staff, extending selection if any",G_CALLBACK (voicedown_cb)},
{"MoveToVoiceUp", NULL, "Move to Voice Up", NULL,"Go to the higher numbered voice on staff without altering selection",G_CALLBACK (movetovoiceup_cb)},
{"MoveToVoiceDown", NULL, "Move to Voice Down", NULL,"Go to the lower numbered voice on this staff without altering selection",G_CALLBACK (movetovoicedown_cb)},
{"AddBefore", NULL, "Add Staff Before", NULL,"Inserts a new staff before the current staff",G_CALLBACK (newstaffbefore)},
{"AddAfter", NULL, "Add Staff After", NULL,"Inserts/Adds a new staff after the current staff",G_CALLBACK (dnm_newstaffafter)},
{"AddInitial", NULL, "Add Initial Staff", NULL,"Inserts a new staff at the top of the score",G_CALLBACK (newstaffinitial)},
{"AddLast", NULL, "Add Last Staff", NULL,"Inserts a new staff at the end of the score",G_CALLBACK (newstafflast)},
{"DeleteBefore", NULL, "Delete Staff Before", NULL,"Deletes the staff before the current staff",G_CALLBACK (delete_staff_before)},
{"DeleteStaff", NULL, "Delete Current Staff", NULL,"Deletes the current staff",G_CALLBACK (delete_staff_current)},
{"DeleteAfter", NULL, "Delete Staff After", NULL,"Deletes the staff after the current staff",G_CALLBACK (delete_staff_after)},
{"AddVoice", NULL, "Add Voice", NULL,"Adds a new voice (part), to the current staff. It is tricky to switch between the voices. Suggest to use merge staffs",G_CALLBACK (dnm_newstaffvoice)},
{"StaffProperties", GTK_STOCK_PROPERTIES, "Staff Properties", NULL,"Change the properties of the current staff",G_CALLBACK (staff_properties_change_cb)},
{"InsertMenu", NULL, "InsertMenu", NULL,"Insert"},
{"Clef", NULL, "Clef", NULL,"Clef"},
{"InitialClef", NULL, "Initial Clef", NULL,"Change the initial clef of the current staff",G_CALLBACK (clef_change_initial)},
{"InsertClef", NULL, "Clef Change", NULL,"Insert/Edit a change of clef at the cursor",G_CALLBACK (clef_change_insert)},
{"Key", NULL, "Keys", NULL,"insert change key signature or set the initial key"},
{"InitialKey", NULL, "Initial Key", NULL,"Set the initial key signature of the current staff",G_CALLBACK (key_change_initial)},
{"InsertKey", NULL, "Key Signature Change", NULL,"Insert/Edit a key change at the cursor position",G_CALLBACK (key_change_insert)},
{"TimeSig", NULL, "Time Signatures", NULL,"Manage the time signature changes and initial value"},
{"InitialTimeSig", NULL, "Inital Time Signature", NULL,"Set the initial time signature of the current staff",G_CALLBACK (timesig_change_initial)},
{"InsertTimeSig", NULL, "Time Signature Change", NULL,"Edit/Insert a time signature change for the current measure",G_CALLBACK (timesig_change_insert)},
{"ChangeNotehead", NULL, "Set Notehead", NULL,"Change the type of notehead for the current note",G_CALLBACK (set_notehead)},
{"InsertStem", NULL, "Auto Stemming", NULL,"Inserts a stem neutral object. After this automatic stem directions are active. You can click on this tag and use Sharpen/StemUp etc commands to change stem direction",G_CALLBACK (stem_directive_insert)},
{"AddVerse", NULL, "Add Lyric Verse", NULL,"Add a verse of lyrics",G_CALLBACK (add_verse)},
{"DeleteVerse", NULL, "Delete Verse", NULL,"Deletes current verse of lyrics from current voice",G_CALLBACK (delete_verse)},
{"EditFiguredBass", NULL, "Insert/Edit Figured Bass", NULL,"Add a bass figure to the current note. Use | sign to split the duration of a note so as to have multiple figures on one note. See Lilypond docs for other notation",G_CALLBACK (figure_insert)},
{"DeleteFiguredBass", NULL, "Delete Figures", NULL,"Delete the figured bass on the current staff",G_CALLBACK (delete_figured_bass)},
{"HideFiguredBass", NULL, "Hide Figures (Print)", NULL,"Hide the figured bass on the current staff on printing",G_CALLBACK (hide_figured_bass)},
{"ShowFiguredBass", NULL, "Show Figures (Print)", NULL,"Show the figured bass on the current staff on printing",G_CALLBACK (show_figured_bass)},
{"EditChords", NULL, "Edit Chord Symbols", NULL,"Allows chord symbols to be added to the current note. E.G.cis:dim7 for c-sharp diminished 7th. See Lilypond docs for notation",G_CALLBACK (fakechord_insert)},
{"InsertDynamic", NULL, "Insert Dynamics", NULL,"Inserts a dynamic marking at the cursor position",G_CALLBACK (insert_dynamic)},
{"EditObject", NULL, "Edit Object", NULL,"Edit the object at the cursor.",G_CALLBACK (edit_object)},
{"EditDirective", NULL, "Edit Directives", NULL,"Edit any directives attached to chord/note at cursor.",G_CALLBACK (edit_object_directive)},
{"EditStaffDirective", NULL, "Edit Staff Directives", NULL,"Edit any directives attached to staff.",G_CALLBACK (edit_staff_directive)},
{"EditVoiceDirective", NULL, "Edit Voice Directives", NULL,"Edit any directives attached to voice.",G_CALLBACK (edit_voice_directive)},
{"EditScoreDirective", NULL, "Edit Score Directives", NULL,"Edit any directives attached to score.",G_CALLBACK (edit_score_directive)},
{"EditMovementDirective", NULL, "Edit Movement Directives", NULL,"Edit any directives attached to movement.",G_CALLBACK (edit_movement_directive)},
{"EditClefDirective", NULL, "Edit Clef Directives", NULL,"Edit any directives attached to clef.",G_CALLBACK (edit_clef_directive)},
{"EditTimesigDirective", NULL, "Edit Time Signature Directives", NULL,"Edit any directives attached to time signature.",G_CALLBACK (edit_timesig_directive)},
{"EditKeysigDirective", NULL, "Edit Key Signature Directives", NULL,"Edit any directives attached to key signature.",G_CALLBACK (edit_keysig_directive)},
{"DeleteDirective", NULL, "Delete a Directive", NULL,"Delete a directive attached to chord/note at cursor.",G_CALLBACK (delete_chord_or_note_directive)},
{"AttachLilyToNote", NULL, "Attach Lilypond to Note", NULL,"Attach or edit attached LilyPond text to the note at the cursor. This can be used for guitar fingerings, cautionary accidentals and much more. See LilyPond documentation.",G_CALLBACK (note_directive)},
{"AttachLilyToChord", NULL, "Attach Lilypond to Chord", NULL,"Attach or edit attached LilyPond text to the chord at the cursor. This can be used for attaching and placing text and much more. See LilyPond documentation.",G_CALLBACK (chord_directive)},
{"InsertBarline", NULL, "Insert Barline", NULL,"Inserts specialized barline at the cursor position. Mostly not working",G_CALLBACK (insert_barline)},
{"NavigationMenu", NULL, "Navigation", NULL,"Getting around the score"},
{"GoToMeasure", NULL, "Go to Measure", NULL,"Opens a dialog for going to a numbered measure",G_CALLBACK (tomeasurenum)},
{"GoToBeginning", GTK_STOCK_GOTO_FIRST, "Go to Beginning", NULL,"Cursor to start of staff/voice, extending selection if any",G_CALLBACK (tohome)},
{"GoToEnd", GTK_STOCK_GOTO_LAST, "Go to End", NULL,"Cursor to end of staff/voice, extending selection if any",G_CALLBACK (toend)},
{"MoveToBeginning", GTK_STOCK_GOTO_FIRST, "Move to Staff/Voice Beginning", NULL,"Cursor to start of staff/voice, without extending selection if any",G_CALLBACK (movetostart)},
{"MoveToEnd", GTK_STOCK_GOTO_LAST, "Move to Staff/Voice End", NULL,"Cursor to end of staff/voice, without extending selection if any",G_CALLBACK (movetoend)},
{"NextMovement", NULL, "Next Movement", NULL,"Go to the next movement",G_CALLBACK (next_movement)},
{"PreviousMovement", NULL, "Previous Movement", NULL,"Go to the previous movement",G_CALLBACK (prev_movement)},
{"DeleteMovement", NULL, "Delete Movement", NULL,"Delete the current movement",G_CALLBACK (delete_movement)},
{"Play", GTK_STOCK_MEDIA_PLAY, "Play", NULL,"Play",G_CALLBACK (ext_midi_playback)},
{"Stop", GTK_STOCK_MEDIA_STOP, "Stop", NULL,"Stop",G_CALLBACK (stop_midi_playback)},
{"PlaybackProperties", GTK_STOCK_PROPERTIES, "Playback Properties", NULL,"Allows you to specify properties used in playing back (midi)",G_CALLBACK (playback_properties_change)},
{"Help", NULL, "Browse Manual", NULL,"Opens a browser on the user manual",G_CALLBACK (browse_manual)},
{"About", NULL, "About", NULL,"Gives the version number etc of this program",G_CALLBACK (about)},
{"MoreMenu", NULL, "More", NULL,"Allows choosing extra commands/menu items from disk"},
{"MoreCommands", NULL, "More Commands", NULL,"Allows choosing standard extra commands/menu items",G_CALLBACK (morecommands)},
{"MyCommands", NULL, "My Commands", NULL,"Allows choosing extra commands/menu items from your own collection of extras",G_CALLBACK (mycommands)},
{"FetchCommands", NULL, "Update Commands from Internet", NULL,"Refreshes the set of commands available from Denemo.org.\nUse More Commands after this has finished",G_CALLBACK (fetchcommands)},
{"OpenRecent", GTK_STOCK_OPEN, "Open Recent", NULL,"Open previously used files"},
{"Import", GTK_STOCK_OPEN, "Import File", NULL,"Import supported file formats"},
{"Export", NULL, "Export As", NULL,"Export File"},
{"ToggleEdit", NULL, "Toggle Edit Mode", NULL,"Toggle between current mode and edit mode",G_CALLBACK (toggle_edit_mode)},
{"ToggleRest", NULL, "Toggle Rest Mode", NULL,"Toggle between note entry and rest entry",G_CALLBACK (toggle_rest_mode)},
{"ToggleRhythm", NULL, "Toggle Audible Feedback", NULL,"Toggle audible feedback on/off",G_CALLBACK (toggle_rhythm_mode)},
{"ClearOverlay", NULL, "Clear Overlay", NULL,"Clear the list of pitches that overlay the notes",G_CALLBACK (clear_overlay)},
{"CreateRhythm", NULL, "Create Snippet", NULL,"Copy selection as music snippet or rhythm pattern for notes to follow as they are entered",G_CALLBACK (create_rhythm_cb)},
{"DeleteRhythm", NULL, "Delete Snippet", NULL,"Delete the selected music snippet/rhythm pattern",G_CALLBACK (delete_rhythm_cb)},
{"ClassicModeNote", NULL, "Classic Mode", NULL,"Moving the cursor and inserting notes or rests there"},
{"SelectNote", NULL, "Select Note", NULL,"Moving the cursor to the nearest ..."},
{"InsertModeNote", NULL, "Insert", NULL,"Actions for notes: inserting, deleting, etc."},
{"StemControl", NULL, "Stem Direction", NULL,"Actions to control the stem up/down"},
{"NoteheadControl", NULL, "Notehead Types", NULL,"Different types of notehead"},
{"TiedNotes", NULL, "Tied Notes", NULL,"Creating Tied Notes"},
{"DisplayEffects", NULL, "Display Effects", NULL,"Effects that are only for the Denemo display,\nnot affecting the printed page."},
{"Articulations", NULL, "Articulations", NULL,"Articulations"},
{"Slurs", NULL, "Slurs", NULL,"Slurs"},
{"GraceNotes", NULL, "Grace Notes", NULL,"Grace Notes"},
{"Score", NULL, "Score", NULL,"Score"},
{"Instruments", NULL, "Instruments", NULL,"Instruments"},
{"Markings", NULL, "Markings", NULL,"Markings"},
{"TextMarks", NULL, "Text/Symbol", NULL,"Text and symbol Markings"},
{"Strings", NULL, "Strings", NULL,"Commands for Stringed Instruments"},
{"InsertNote", NULL, "Note Insertion", NULL,"Inserting the note ..."},
{"Lyrics", NULL, "Lyrics", NULL,"Lyrics"},
{"Directives", NULL, "Directives", NULL,"Standalone Directives"},
{"Navigation", NULL, "Navigation", NULL,"Moving around the piece"},
{"NoteEntry", NULL, "Insert Note", NULL,"Entering notes"},
{"Articulation", NULL, "Articulation", NULL,"Various expressive marks"},
{"Edit", NULL, "Edit", NULL,"Editing"},
{"Measure", NULL, "Measure", NULL,"Manipulating measures"},
{"Staff", NULL, "Staff", NULL,"Commands for staffs"},
{"Playback", NULL, "Playback", NULL,"Playing the music through midi file"},
{"SelectDuration", NULL, "Select Duration", NULL,"Changing the prevailing duration or rhythm pattern"},
{"EditModeNote", NULL, "Append/Edit", NULL,"Appending, Changing, and deleting notes"},
{"EditNote", NULL, "Append/Edit Note", NULL,"Appending or Editing notes"},
{"EditDuration", NULL, "Append/Insert Duration", NULL,"Appending or Editing durations"},
{"Cursor", NULL, "Cursor", NULL,"Moving the cursor"},
{"CursorToNote", NULL, "Cursor to Note", NULL,"Moving the cursor to note positions"},
{"ClefMenu", NULL, "Clefs", NULL,"Insert/change clef Set initial clef"},
{"ChordMenu", NULL, "Chords", NULL,"Adding stuff for chords"},
{"ChordNoteMenu", NULL, "Add Note", NULL,"Adding notes to make chords"},
{"ChordSymbols", NULL, "Chords Symbols", NULL,"Adding Chord Symbols over music"},
{"FiguredBass", NULL, "Figured Bass", NULL,"Adding Figured Bass Figures"},
{"MeasureMenu", NULL, "Measures", NULL,"Measures: adding, deleting, navigating etc"},
{"Insert", NULL, "Insert", NULL,"Inserting notes, measures, staffs, keysigs etc"},
{"InsertStaff", NULL, "Add Staff", NULL,"Insert a Staff relative to current staff"},
{"InsertMovement", NULL, "Insert Movement", NULL,"Insert a Movement relative to current movement"},
{"InsertDuration", NULL, "Insert at Cursor", NULL,"Inserting notes of a given duration at cursor note height"},
{"Change", NULL, "Change", NULL,"Changing properties of notes, measures, staffs, keysigs etc"},
{"ModelessNote", NULL, "Notes/Durations", NULL,"Modeless actions on notes/rests"},
{"NotesRests", NULL, "Notes/Rests", NULL,"Actions for notes/rests"},
{"RestEntry", NULL, "Rest Insertion", NULL,"Modeless entry of rests"},
{"EditDirectivesMenu", NULL, "Edit Directive(s)", NULL,"Editing directives"},
{"ChangeNote", NULL, "Change Note", NULL,"Changing the note at the cursor to the nearest ..."},
{"ChangeDuration", NULL, "Edit Duration", NULL,"Changes the duration of the current note"},
{"ChangeRest", NULL, "Change Rest", NULL,"Changes the duration of the current rest"},
{"ExpressionMarks", NULL, "Expression Marks", NULL,"Dynamics, staccato, slurs, ties and other expressive marks"},
{"Dynamics", NULL, "Dynamics", NULL,"Dynamic markings"},
{"Ornaments", NULL, "Ornaments", NULL,"grace notes etc"},
{"Other", NULL, "Other", NULL,"Lyrics, chord symbols, figured basses etc"},
{"Others", NULL, "Others", NULL,"Less used actions"},
{"Favorites", NULL, "Favorites", NULL,"Customized LilyPond inserts. Store often-used inserts here labelled with what they do"},
{"AddFavorite", NULL, "Add Favorite", NULL,"Add a custom LilyPond insert to favorites menu"},
{"Tuplets", NULL, "Tuplets", NULL,"Entering triplets and other tuplets"},
{"Delete", NULL, "Delete", NULL,"Deleting notes, measures staffs keysigs etc"},

#define NOTECHAR0 "\xF0\x9D\x85\x9D"

#define RESTCHAR0 "\xF0\x9D\x84\xBB"

#define NOTE0 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x9D</span>"

#define REST0 "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBB</span>"
{"0", "NULL", NOTE0, NULL, "Edits the note at the cursor to have the duration 1/2^0, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur0)},
{"Change0", "NULL", NOTE0, NULL, "Change current note to a "NOTE0,
G_CALLBACK (ChangeDur0)},
{"ChangeRest0", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest0)},
{"Insert0", NULL, "Insert a "NOTE0"", NULL, "Inserts a "NOTE0" at cursor position\nSets prevailing rhythm to "NOTE0,
G_CALLBACK (InsertDur0)},
{"InsertRest0", NULL, "Insert a "REST0"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE0,
G_CALLBACK (InsertRest0)},
{"Set0", NULL, "Set Duration to "NOTE0"", NULL, "Sets prevailing rhythm to "NOTE0,
G_CALLBACK (SetDur0)},

#define NOTECHAR1 "\xF0\x9D\x85\x9E"

#define RESTCHAR1 "\xF0\x9D\x84\xBC"

#define NOTE1 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x9E</span>"

#define REST1 "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBC</span>"
{"1", "NULL", NOTE1, NULL, "Edits the note at the cursor to have the duration 1/2^1, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur1)},
{"Change1", "NULL", NOTE1, NULL, "Change current note to a "NOTE1,
G_CALLBACK (ChangeDur1)},
{"ChangeRest1", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest1)},
{"Insert1", NULL, "Insert a "NOTE1"", NULL, "Inserts a "NOTE1" at cursor position\nSets prevailing rhythm to "NOTE1,
G_CALLBACK (InsertDur1)},
{"InsertRest1", NULL, "Insert a "REST1"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE1,
G_CALLBACK (InsertRest1)},
{"Set1", NULL, "Set Duration to "NOTE1"", NULL, "Sets prevailing rhythm to "NOTE1,
G_CALLBACK (SetDur1)},

#define NOTECHAR2 "\xF0\x9D\x85\x9F"

#define RESTCHAR2 "\xF0\x9D\x84\xBD"

#define NOTE2 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x9F</span>"

#define REST2 "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBD</span>"
{"2", "NULL", NOTE2, NULL, "Edits the note at the cursor to have the duration 1/2^2, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur2)},
{"Change2", "NULL", NOTE2, NULL, "Change current note to a "NOTE2,
G_CALLBACK (ChangeDur2)},
{"ChangeRest2", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest2)},
{"Insert2", NULL, "Insert a "NOTE2"", NULL, "Inserts a "NOTE2" at cursor position\nSets prevailing rhythm to "NOTE2,
G_CALLBACK (InsertDur2)},
{"InsertRest2", NULL, "Insert a "REST2"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE2,
G_CALLBACK (InsertRest2)},
{"Set2", NULL, "Set Duration to "NOTE2"", NULL, "Sets prevailing rhythm to "NOTE2,
G_CALLBACK (SetDur2)},

#define NOTECHAR3 "\xF0\x9D\x85\xA0"

#define RESTCHAR3 "\xF0\x9D\x84\xBE"

#define NOTE3 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA0</span>"

#define REST3 "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBE</span>"
{"3", "NULL", NOTE3, NULL, "Edits the note at the cursor to have the duration 1/2^3, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur3)},
{"Change3", "NULL", NOTE3, NULL, "Change current note to a "NOTE3,
G_CALLBACK (ChangeDur3)},
{"ChangeRest3", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest3)},
{"Insert3", NULL, "Insert a "NOTE3"", NULL, "Inserts a "NOTE3" at cursor position\nSets prevailing rhythm to "NOTE3,
G_CALLBACK (InsertDur3)},
{"InsertRest3", NULL, "Insert a "REST3"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE3,
G_CALLBACK (InsertRest3)},
{"Set3", NULL, "Set Duration to "NOTE3"", NULL, "Sets prevailing rhythm to "NOTE3,
G_CALLBACK (SetDur3)},

#define NOTECHAR4 "\xF0\x9D\x85\xA1"

#define RESTCHAR4 "\xF0\x9D\x84\xBF"

#define NOTE4 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA1</span>"

#define REST4 "<span font_desc=\"Denemo\">\xF0\x9D\x84\xBF</span>"
{"4", "NULL", NOTE4, NULL, "Edits the note at the cursor to have the duration 1/2^4, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur4)},
{"Change4", "NULL", NOTE4, NULL, "Change current note to a "NOTE4,
G_CALLBACK (ChangeDur4)},
{"ChangeRest4", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest4)},
{"Insert4", NULL, "Insert a "NOTE4"", NULL, "Inserts a "NOTE4" at cursor position\nSets prevailing rhythm to "NOTE4,
G_CALLBACK (InsertDur4)},
{"InsertRest4", NULL, "Insert a "REST4"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE4,
G_CALLBACK (InsertRest4)},
{"Set4", NULL, "Set Duration to "NOTE4"", NULL, "Sets prevailing rhythm to "NOTE4,
G_CALLBACK (SetDur4)},

#define NOTECHAR5 "\xF0\x9D\x85\xA2"

#define RESTCHAR5 "\xF0\x9D\x85\x80"

#define NOTE5 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA2</span>"

#define REST5 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x80</span>"
{"5", "NULL", NOTE5, NULL, "Edits the note at the cursor to have the duration 1/2^5, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur5)},
{"Change5", "NULL", NOTE5, NULL, "Change current note to a "NOTE5,
G_CALLBACK (ChangeDur5)},
{"ChangeRest5", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest5)},
{"Insert5", NULL, "Insert a "NOTE5"", NULL, "Inserts a "NOTE5" at cursor position\nSets prevailing rhythm to "NOTE5,
G_CALLBACK (InsertDur5)},
{"InsertRest5", NULL, "Insert a "REST5"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE5,
G_CALLBACK (InsertRest5)},
{"Set5", NULL, "Set Duration to "NOTE5"", NULL, "Sets prevailing rhythm to "NOTE5,
G_CALLBACK (SetDur5)},

#define NOTECHAR6 "\xF0\x9D\x85\xA3"

#define RESTCHAR6 "\xF0\x9D\x85\x81"

#define NOTE6 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA3</span>"

#define REST6 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x81</span>"
{"6", "NULL", NOTE6, NULL, "Edits the note at the cursor to have the duration 1/2^6, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur6)},
{"Change6", "NULL", NOTE6, NULL, "Change current note to a "NOTE6,
G_CALLBACK (ChangeDur6)},
{"ChangeRest6", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest6)},
{"Insert6", NULL, "Insert a "NOTE6"", NULL, "Inserts a "NOTE6" at cursor position\nSets prevailing rhythm to "NOTE6,
G_CALLBACK (InsertDur6)},
{"InsertRest6", NULL, "Insert a "REST6"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE6,
G_CALLBACK (InsertRest6)},
{"Set6", NULL, "Set Duration to "NOTE6"", NULL, "Sets prevailing rhythm to "NOTE6,
G_CALLBACK (SetDur6)},

#define NOTECHAR7 "\xF0\x9D\x85\xA4"

#define RESTCHAR7 "\xF0\x9D\x85\x82"

#define NOTE7 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA4</span>"

#define REST7 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x82</span>"
{"7", "NULL", NOTE7, NULL, "Edits the note at the cursor to have the duration 1/2^7, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur7)},
{"Change7", "NULL", NOTE7, NULL, "Change current note to a "NOTE7,
G_CALLBACK (ChangeDur7)},
{"ChangeRest7", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest7)},
{"Insert7", NULL, "Insert a "NOTE7"", NULL, "Inserts a "NOTE7" at cursor position\nSets prevailing rhythm to "NOTE7,
G_CALLBACK (InsertDur7)},
{"InsertRest7", NULL, "Insert a "REST7"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE7,
G_CALLBACK (InsertRest7)},
{"Set7", NULL, "Set Duration to "NOTE7"", NULL, "Sets prevailing rhythm to "NOTE7,
G_CALLBACK (SetDur7)},

#define NOTECHAR8 "\xF0\x9D\x85\xA5"

#define RESTCHAR8 "\xF0\x9D\x85\x83"

#define NOTE8 "<span font_desc=\"Denemo\">\xF0\x9D\x85\xA5</span>"

#define REST8 "<span font_desc=\"Denemo\">\xF0\x9D\x85\x83</span>"
{"8", "NULL", NOTE8, NULL, "Edits the note at the cursor to have the duration 1/2^8, or \n(if appending) appends such a duration -  to be completed with a note name. Changes prevailing duration",
G_CALLBACK (Dur8)},
{"Change8", "NULL", NOTE8, NULL, "Change current note to a "NOTE8,
G_CALLBACK (ChangeDur8)},
{"ChangeRest8", NULL, "Change duration", NULL, "Change duration of current rest",
G_CALLBACK (ChangeRest8)},
{"Insert8", NULL, "Insert a "NOTE8"", NULL, "Inserts a "NOTE8" at cursor position\nSets prevailing rhythm to "NOTE8,
G_CALLBACK (InsertDur8)},
{"InsertRest8", NULL, "Insert a "REST8"rest", NULL, "Inserts a rest at cursor position\nSets prevailing rhythm to "NOTE8,
G_CALLBACK (InsertRest8)},
{"Set8", NULL, "Set Duration to "NOTE8"", NULL, "Sets prevailing rhythm to "NOTE8,
G_CALLBACK (SetDur8)},
 {"InsertA", NULL, "Insert A", NULL, "Inserts note A before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertA)},
 {"AddNoteA", NULL, "Insert A After", NULL, "Inserts note A after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteA)},
 {"AddA", NULL, "Add A to Chord", NULL, "Adds note A to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddA)},
  {"ChangeToA", NULL, "Change current note to A", NULL, "Changes current note to the A nearest cursor or (if no current note) inserts the note A\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToA)},
  {"MoveToA", NULL, "Move cursor to step A", NULL, "Moves the cursor to the A nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToA)},
 {"InsertB", NULL, "Insert B", NULL, "Inserts note B before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertB)},
 {"AddNoteB", NULL, "Insert B After", NULL, "Inserts note B after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteB)},
 {"AddB", NULL, "Add B to Chord", NULL, "Adds note B to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddB)},
  {"ChangeToB", NULL, "Change current note to B", NULL, "Changes current note to the B nearest cursor or (if no current note) inserts the note B\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToB)},
  {"MoveToB", NULL, "Move cursor to step B", NULL, "Moves the cursor to the B nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToB)},
 {"InsertC", NULL, "Insert C", NULL, "Inserts note C before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertC)},
 {"AddNoteC", NULL, "Insert C After", NULL, "Inserts note C after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteC)},
 {"AddC", NULL, "Add C to Chord", NULL, "Adds note C to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddC)},
  {"ChangeToC", NULL, "Change current note to C", NULL, "Changes current note to the C nearest cursor or (if no current note) inserts the note C\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToC)},
  {"MoveToC", NULL, "Move cursor to step C", NULL, "Moves the cursor to the C nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToC)},
 {"InsertD", NULL, "Insert D", NULL, "Inserts note D before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertD)},
 {"AddNoteD", NULL, "Insert D After", NULL, "Inserts note D after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteD)},
 {"AddD", NULL, "Add D to Chord", NULL, "Adds note D to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddD)},
  {"ChangeToD", NULL, "Change current note to D", NULL, "Changes current note to the D nearest cursor or (if no current note) inserts the note D\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToD)},
  {"MoveToD", NULL, "Move cursor to step D", NULL, "Moves the cursor to the D nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToD)},
 {"InsertE", NULL, "Insert E", NULL, "Inserts note E before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertE)},
 {"AddNoteE", NULL, "Insert E After", NULL, "Inserts note E after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteE)},
 {"AddE", NULL, "Add E to Chord", NULL, "Adds note E to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddE)},
  {"ChangeToE", NULL, "Change current note to E", NULL, "Changes current note to the E nearest cursor or (if no current note) inserts the note E\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToE)},
  {"MoveToE", NULL, "Move cursor to step E", NULL, "Moves the cursor to the E nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToE)},
 {"InsertF", NULL, "Insert F", NULL, "Inserts note F before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertF)},
 {"AddNoteF", NULL, "Insert F After", NULL, "Inserts note F after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteF)},
 {"AddF", NULL, "Add F to Chord", NULL, "Adds note F to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddF)},
  {"ChangeToF", NULL, "Change current note to F", NULL, "Changes current note to the F nearest cursor or (if no current note) inserts the note F\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToF)},
  {"MoveToF", NULL, "Move cursor to step F", NULL, "Moves the cursor to the F nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToF)},
 {"InsertG", NULL, "Insert G", NULL, "Inserts note G before note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (InsertG)},
 {"AddNoteG", NULL, "Insert G After", NULL, "Inserts note G after note at cursor\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
  G_CALLBACK (AddNoteG)},
 {"AddG", NULL, "Add G to Chord", NULL, "Adds note G to chord at cursor\nCursor determines which octave",
  G_CALLBACK (AddG)},
  {"ChangeToG", NULL, "Change current note to G", NULL, "Changes current note to the G nearest cursor or (if no current note) inserts the note G\nCursor determines which octave\nNote is inserted in the prevailing rhythm",
   G_CALLBACK (ChangeToG)},
  {"MoveToG", NULL, "Move cursor to step G", NULL, "Moves the cursor to the G nearest cursor\nCurrent cursor position determines which octave.",
   G_CALLBACK (MoveToG)},
